<?php
class Album {
  public function __construct($album, $images, $fish) {
    $this->name = $album;
    $this->title = PRETTYTITLES === true ? makePrettyTitle($album) : $album;
    foreach ($images as $image) {
      if (is_dir($image)) continue;
      $baseimg = basename($image);
      $thumb = "thumbs/$album/" . strip_ext($baseimg) . 'jpg';
      if (is_readable($thumb) || isValidImage($image)) $this->images[] = $baseimg;
    }
    $this->imagecount = count($this->images);
    if (!$fish || $fish === $album) $this->addCaptions($album);
    $this->password = Shampoo::get_password($album);
  }

  public function addCaptions($album) {
    $file = @file($album . '/' . $album . '.txt', FILE_SKIP_EMPTY_LINES);
    if ($file) {
      foreach ($file as $line) {
        $bits = parseLine($line);
        if ($bits) $data[$bits[0]] = $bits[1];
      }
    }
    foreach ((array)$this->images as $image) { // cast to array if $this->images is empty to stop foreach throwing an error
      if ($data[$image]) $this->captions[$image] = ampquot($data[$image]);
      elseif (FNORD === true && function_exists('fnord')) $this->captions[$image] = fnord();
      else $this->captions[$image] = $image;
    }
  }
}

class Shampoo {
  public function __construct($fish) {
    $ignore = $GLOBALS['folderstoignore'];
    $albums = (array)glob("*", GLOB_ONLYDIR);
    if (is_array($ignore)) $albums = array_diff($albums, $ignore);
    if (!in_array($fish, $albums)) unset($fish);
    foreach ($albums as $album) {
      if (is_readable($album)) {
        $this->albumlist[] = $album;
        $images = (array)glob("$album/*.*");
        $this->$album = new Album($album, $images, $fish);
      }
    }
    if ($fish && $this->$fish) $this->album = $fish;
    $this->albumcount = count($this->albumlist);
    if (is_readable(INDEX)) {
      $desc = false;
      foreach (file(INDEX, FILE_SKIP_EMPTY_LINES) as $line) {
        $bits = parseLine($line);
        if (empty($bits)) continue;
        if (count($bits) === 1) {
          if ($bits[0] === '***') {
            $desc = !$desc;
            continue;
          }
          if (!$desc) {
            $this->title = $bits[0];
          }
          else {
            $this->introtext .= $bits[0] . "\n";
          }
        }
        elseif ($this->$bits[0]) {
          $this->$bits[0]->title = $bits[1] ? trim(htmlspecialchars($bits[1], ENT_COMPAT)) : trim($bits[0]);
          $this->$bits[0]->description = trim($bits[2]);
        }
      }
    }
    switch(SORTBY) {
      case 'title': usort($this->albumlist, array($this, 'titleSort')); break;
      case 'name': sort($this->albumlist); break;
      case 'date': usort($this->albumlist, 'dateSort'); break;
    }
  }

  public function makeMenu() {
    foreach ($this->albumlist as $album) {
      $html .= '        <li><a href="' . $album . '/"';
      if ($this->album === $album || $this->$album->password) {
        $html .= ' class="';
        if ($this->album === $album) $html .= 'current ';
        if ($this->$album->password) $html .= 'private';
        $html = rtrim($html) . '"';
      }
      $pic = $this->$album->imagecount === 1 ? ' picture' : ' pictures';
      $html .= ' title="' . $this->$album->imagecount . $pic . '">' . $this->$album->title;
      if ($this->$album->password && !$_SESSION['access'][$album]) $html .= ' <img src="img/lock.gif" width="8" height="10" title="Private album - you will be asked for a password" alt="(Private)">';
      $html .= "</a></li>\n";
    }
    $this->register_var('albumlist', ltrim($html));
  }

  public function showList() {
    $this->register_var('content', "<div>\n    " . $this->introtext . "\n    </div>\n    <ul id=\"albumarray\">\n");
    $listcount = 0;
    foreach ($this->albumlist as $key => $album) {
      if (!$_SESSION['admin'] && $this->$album->password && !$_SESSION['access'][$album]) continue;
      $icount = $this->$album->imagecount;
      $desc = $this->$album->description;
      $images = (array)$this->$album->images; // cast to array so shuffle won't fail on this being null if no images in album
      $title = $this->$album->title;
      if (RANDOM === true) shuffle($images);
      $cls = $listcount % 2 ? ' class="odd"' : '';
      $listcount++;
      $howmany = $icount === 1 ? 'One picture' : "$icount pictures";
      $buff = "      <li$cls>
        <h3>
          <a href=\"{$this->$album->name}/\" title=\"Open $title\">$title</a>
          <a href=\"{$this->$album->name}/\" title=\"Open $title\">Open album</a>
        </h3>
        <p class=\"imagecount\">$howmany</p>\n";
      if ($icount > 0) {
        $buff .= "        <div>\n";
        $classextra = '';
        foreach ($images as $key => $image) {
          $caption = $this->$album->captions[$image];
          $thumbpath = createThumb($album, $image);
          if (!file_exists($thumbpath)) continue;
          $title = htmlspecialchars(strip_tags($caption));
          if ($key > 3) $classextra = ' thumbhide';
          $size = @getimagesize($thumbpath);
          $buff .= '          <a href="' . "$album/" . fakeUrl($image) . '" title="' . $title . '" rel="' . $album . '" class="thickbox' . $classextra . '"><img src="' . $thumbpath . '" alt="' . truncate($title) . '" ' . $size[3] . "></a>\n          <p>$caption</p>\n";
          if ($key === PERALBUM - 1) break;
        }
        $buff .= "        </div>\n";
      }
      else $buff .= "        <p>This album is devoid of images!</p>\n";
      if (DESCRIPTIONS === true && $desc) {
        $c = strlen($desc) > 750 ? ' columnar' : '';
        $buff .= "        <p class=\"description$c\">$desc</p>\n";
      }
      $this->register_var('content', $buff . "      </li>\n");
    }
    $this->register_var('content', "    </ul>\n");
  }

  public function albumate($fish) {
    $album = $this->album;
    if ($this->$album->password && !$_SESSION['admin']) {
      if ($_POST['password']) {
        if (!$_SESSION['access']) $_SESSION['access'] = array();
        if (utf8_encode(hash('tiger192,4', $_POST['password'])) === $this->$album->password) $_SESSION['access'][$album] = true;
        else $pass = false;
      }
      if (!$_SESSION['access'][$album]) {
        $this->makeHeadLink('up', './', 'List of albums');
        $this->register_var('header', '<a href="./" title="Front page">' . $this->title . '</a> &raquo; <span>' . $this->{$this->album}->title . '</span>');
        $html = "  <form action=\"$album/{$fish[1]}\" method=\"post\" id=\"blockade\">\n";
        if ($pass === false) $html .= '    <p class="error">Wrong password entered. Please note it is case sensitive.</p>'."\n";
        $html .= "    <p>This is a private album. To see it you must enter a password.</p>\n    <label>Password: <input type=\"password\" name=\"password\"></label>\n    <input value=\"Go\" type=\"submit\">\n  </form>\n";
        $this->register_var('content', $html);
        return;
      }
    }
    $icount = $this->$album->imagecount;
    if ($_POST['imageno'] && is_numeric($_POST['imageno'])) {
      if ($_POST['imageno'] <= $icount) {
        header('Location:' . BASE . $this->album . '/' . fakeUrl($this->$album->images[$_POST['imageno'] - 1]));
        exit;
      }
    }
    $this->makeHeadLink('top', './', 'List of albums');
    $desc = $this->$album->description;
    $howmany = $icount === 1 ? 'One picture' : "$icount pictures";
	  $this->register_var('content', "<p class=\"imagecount\">$howmany</p>\n");
    if ($desc) {
      if (strlen($desc) > 350) $c = ' columnar';
      $this->register_var('content', "    <p class=\"description$c\">$desc</p>\n");
    }
    if (is_numeric($fish[1]) || empty($fish[1])) $this->thumbView((int)$fish[1]);
    else $this->singleImageView($fish[1]);
    $this->gen($fish[1]);
  }

  public function thumbView($page) {
    $this->makeHeadLink('up', './', 'List of albums');
    $this->register_var('header', '<a href="./" title="Front page">' . $this->title . '</a> &raquo; <span>' . $this->{$this->album}->title . '</span>');
    $this->thumbrange = $GLOBALS['thumbsdisplayed'];
    if ($this->{$this->album}->imagecount === 0) {
      $this->toshow = DEFAULTTHUMBS;
      $this->currentpage = 1;
      $this->options(true);
      $this->register_var('content', "    <p>Nothing to see here!</p>\n    <p><a href=\"./\">&laquo; Back to album listing</a></p>\n");
      return;
    }
    $ppp = $_POST['perpage'];
    $spp = $_SESSION['perpage'];
    $scpp = $_SESSION['customperpage'];
    $this->oldtoshow = $this->toshow = $spp ? ($spp === 'custom' && $scpp ? $scpp : $spp) : DEFAULTTHUMBS;
    if ($ppp && in_array($ppp, $this->thumbrange)) {
      if ($ppp === 'custom') {
        $pcpp = (int)$_POST['customperpage'];
        if (!in_array($pcpp, $this->thumbrange) && $pcpp > 0) {
          $_SESSION['perpage'] = $ppp;
          $this->toshow = $_SESSION['customperpage'] = $pcpp;
        }
        elseif ($pcpp > 0) {
          $this->toshow = $_SESSION['perpage'] = $pcpp;
          unset($_SESSION['customperpage']);
        }
      }
      else $this->toshow = $_SESSION['perpage'] = (int)$ppp;
    }
    if ($this->toshow < 1) $this->toshow = DEFAULTTHUMBS;
    $this->pages = (int)ceil($this->{$this->album}->imagecount / $this->toshow);
    $page = min(max($page, 1), $this->pages);
    $this->startmark = $this->oldtoshow * ($page - 1);
    $this->currentpage = (int)ceil($this->startmark / $this->toshow) + 1;
    $this->startmark = $this->toshow * ($this->currentpage - 1);
    $this->endmark = min($this->{$this->album}->imagecount, $this->startmark + $this->toshow);
    $this->options();
    if ($this->currentpage > 1) {
      $this->makePageNavLink('prev', $this->currentpage-1);
      $this->makeHeadLink('first', $this->album . '/1/', 'First page of thumbs');
      $this->makeHeadLink('prev', $this->album . '/' . $this->currentpage-1 . '/', 'Page '. ($this->currentpage-1));
    }
    if ($this->currentpage < $this->pages) {
      $this->makePageNavLink('next', $this->currentpage+1, $this->pages);
      $this->makeHeadLink('next', $this->album . '/' . $this->currentpage+1 . '/', 'Page '. ($this->currentpage+1));
      $this->makeHeadLink('last', $this->album . '/' . $this->pages . '/', 'Last page of thumbs');
    }
    $buff = '    <ul id="thumbarray">';
    for ($i = $this->startmark; $i < $this->endmark ; $i++) {
      $image = $this->{$this->album}->images[$i];
 	    $thumbpath = createThumb($this->album, $image);
      if (!file_exists($thumbpath)) continue;
      $buff .= "\n      <li>\n        <a href=\"" . $this->album . '/' . fakeUrl($image) . '"';
      $caption = $this->{$this->album}->captions[$image];
      $title = strip_tags($caption);
      $alt = truncate($title);
      $caption = truncate($caption);
      $size = @getimagesize($thumbpath);
      $buff .= ' class="thickbox" title="' . $title . '">
        <span><img src="' . $thumbpath . "\" alt=\"$alt\" $size[3]></span>
        <span class=\"caption\">$caption</span>\n";
      $datetime = getExif($this->album . "/$image", true);
      if ($datetime) $buff .= "        <span class=\"exif\">Taken $datetime</span>\n";
      $buff .= "        </a>\n";
      $buff .= "      </li>\n";
    }
    $this->register_var('content', "$buff    </ul>\n");
  }

  public function options($empty = false) {
    $buff = '    <div id="bar">' . "\n";
    $imagecount = $this->{$this->album}->imagecount;
    if ($empty) {
      $buff .= "      <span>This album is empty</span>\n";
    }
    elseif ($this->pages > 1) {
      $buff .= "      <span><strong>&nbsp;</strong>Page</span>\n      <div>\n        <ol";
      if ($this->pages > 9) $buff .= ' class="doubledigits"';
      $buff .= ">\n";
      for ($i = 1; $i <= $this->pages; $i++) {
        $c = $i === $this->currentpage ? ' class="current"' : '';
        $buff .= "          <li$c><a href=\"{$this->album}/$i/\">$i</a></li>\n";
      }
      $buff .= "        </ol>\n      </div>\n";
      $info = $this->startmark + 1 === $imagecount ? "<strong>the last lonely image</strong>" : (strval($this->toshow) === "1" ? "image <strong>" . ($this->starmark + 1) . "</strong>" : "images <strong>" . ($this->startmark + 1) . "</strong> to <strong>{$this->endmark}</strong>");
      $buff .= "      <span>of <strong>{$this->pages}</strong> showing $info</span>\n";
    }
    else {
      $buff .= '      <span>';
      if ($imagecount === 1) $buff .= 'The one and only image in this album';
      else $buff .= "All <strong>$imagecount</strong> images displayed";
      $buff .= "</span>\n";
    }
    $buff .= '      <form method="post" action="' . "{$this->album}/{$this->currentpage}/\">
        <fieldset>
          <legend>Settings</legend>\n";
    $buff .= '          <label for="perpage">Thumbs per page:</label>
          <select name="perpage" id="perpage"';
    if ($_SESSION['perpage'] === 'custom') $buff .= ' class="custom"';
    $buff .= ">\n";
    foreach ($this->thumbrange as $o) {
      if ($o === 'custom') continue;
      $buff .= "            <option value=\"$o\"";
      $perpage = in_array($this->toshow, $this->thumbrange) ? $this->toshow : 'custom';
      $buff .= (string)$o === (string)$perpage ? ' selected="selected">' : '>';
      $buff .= $o;
      $buff .= "</option>\n";
    }
    $buff .= '          </select>' . "\n";
    if ((string)$perpage === 'custom') $buff .= '          <input type="text" maxlength="3" value="'.$_SESSION['customperpage'].'" name="customperpage">' . "\n";
    $buff .= '          <input type="image" src="img/right.png" alt="Go">
        </fieldset>
      </form>
    </div>' . "\n";
  $this->register_var('content', $buff);
  }

  public function makePageNavLink($dir, $index, $pages = false) {
    if (!$pages) {
      $anchortitle = $index > 1 ? 'Previous' : 'First';
      $anchortitle .= " {$this->toshow} images";
    }
    else {
      if ($index === $pages) {
        $dregs = $this->{$this->album}->imagecount - ($this->toshow * $index + 1);
        $anchortitle = $dregs > 1 ? "Last $dregs images" : "Last page, last image";
      }
      else $anchortitle = "Next {$this->toshow} images";
    }
    if ($index > 9) $nd = ' doubledigits';
    $buff = '  <div class="pagination ' . $dir . $nd . '">
    <a href="' . $this->album . '/' . $index . '/" title="' . $anchortitle . '">' . "\n";
    while ($s++ < 2) {
      if ($s === 2) $dir = 'page';
      $buff .= "      <span>";
      foreach (str_split($dir) as $letter) $buff .= "<span>$letter</span>";
      $buff .= "</span>\n";
    }
    $this->register_var('content', $buff . "    </a>\n    <a href=\"{$this->album}/$index/\">$index</a>\n  </div>\n");
  }

  public function singleImageView($image) {
    $images = $this->{$this->album}->images;
    $captions = $this->{$this->album}->captions;
    $imagecount = $this->{$this->album}->imagecount;
    $albumname = $this->{$this->album}->name;
    $this->makeHeadLink('up', $albumname . '/', $this->{$this->album}->title);
    $this->register_var('header', '<a href="./" title="Front page">' . $this->title . '</a> &raquo; <a href="' . $albumname . '/">' . $this->{$this->album}->title . '</a>');
    $current = $image ? (int)array_search(realUrl($image), $images) : 0;
    $imagepath = $this->album . '/' . $images[$current];
    $buff = "    <div id=\"view\">\n";
    $buff .= "      <form action=\"{$_SERVER['REQUEST_URI']}/\" method=\"post\">\n        <fieldset>\n          <legend>Browse by position</legend>\n          <label>Go to # <input type=\"text\" name=\"imageno\"> / $imagecount</label>\n          <input type=\"image\" src=\"img/right.png\" alt=\"Go\">\n        </fieldset>\n      </form>\n";
    $buff .= "      <p>\n";
    if ($current > 0) {
      $this->makeHeadLink('first', $albumname . '/' . fakeUrl($images[0]), ampquot(strip_tags($captions[$images[0]])));
      $this->makeHeadLink('prev', $albumname . '/' . fakeUrl($images[$current-1]), ampquot(strip_tags($captions[$images[$current-1]])));
      $buff .= '        <a href="' . $this->album . '/' . fakeUrl($images[$current-1]) . '" title="' . strip_tags($captions[$images[$current-1]]) . "\">&laquo; Prev</a>\n";
    }
    $buff .= "        Image " . ($current + 1) . " of $imagecount\n";
    if ($current < $imagecount - 1) {
      $this->makeHeadLink('next', $this->album . '/' . fakeUrl($images[$current+1]), ampquot(strip_tags($captions[$images[$current+1]])));
      $this->makeHeadLink('last', $this->album . '/' . fakeUrl($images[$imagecount-1]), ampquot(strip_tags($captions[$images[$imagecount-1]])));
      $buff .= '        <a href="' . $this->album . '/' . fakeUrl($images[$current+1]) . '" title="' . strip_tags($captions[$images[$current+1]]) . "\">Next &raquo;</a>\n";
    }
    $buff .= "      </p>\n      <div>\n";
    $size = @getimagesize($imagepath);
    if (is_array($size)) { // it's an image
      $buff .= '        <img src="' . $this->album . '/' . $images[$current] . '" alt="' . strip_tags($captions[$images[$current]]) . '" ' . $size[3] . '>' . "\n";
      if (EXIF === true) {
        $exifdata = getExif($imagepath, false);
        if ($exifdata) {
          $buff .= '      <div class="exif">' . "\n" .
                   '        <h4>EXIF data</h4>' . "\n" .
                   "        <dl>\n";
          foreach ($exifdata as $label => $value) {
            $buff .= "          <dt>$label:</dt>\n" .
                     "          <dd>$value</dd>\n";
          }
          $buff .= "        </dl>\n" .
                   "      </div>\n";
        }
      }
    }
    else { // it must be a video
      $v = isValidVideo($imagepath, 0);
      if ($v === true) { // ffmpeg is not installed, so no dimensions provided
        $width = '640';
        $height = '385'; // same as youtube
        $scaling = ', &quot;scaling&quot;: &quot;fit&quot;';
      }
      elseif (is_array($v)) list($width, $height) = $v;
      $buff .= '        <object id="player_api" data="js/flowplayer.swf" type="application/x-shockwave-flash" width="' . $width . '" height="' . $height . '">' . "\n" .
               '          <param name="allowfullscreen" value="true">' . "\n" .
               '          <param name="allowscriptaccess" value="always">' . "\n" .
               '          <param name="quality" value="high">' . "\n" .
               '          <param name="bgcolor" value="#000000">' . "\n" .
               '          <param name="flashvars" value="config={&quot;clip&quot;:{&quot;url&quot;:&quot;' . $images[$current] . '&quot;' . $scaling . '}}">' . "\n" .
               '          <p>Videos on this site are played via a Flash video player. If you don\'t want to install the Flash plugin, you can <a href="' . $imagepath . '">download the video</a> instead.</p>' . "\n" .
               '        </object>' . "\n";
    }
    $buff .= "      </div>\n      <p class=\"caption\">" . $captions[$images[$current]] . '</p>
      <div class="thumbs">
        <ul>' . "\n";
    for ($i = max($current-2, 0), $j = min($current+3, $imagecount); $i < $j; $i++) {
      $image = $images[$i];
      $thumbpath = createThumb($albumname, $image);
      if (!file_exists($thumbpath)) continue;
      $imagehref = fakeUrl($image);
      $actclass = $i === $current ? ' class="active"' : null;
      $title = strip_tags($captions[$image]);
      $buff .= "          <li$actclass>
            <a href=\"" . $this->album . "/$imagehref\" title=\"$title\"><img src=\"$thumbpath\" alt=\"" . truncate($title) . "\"></a>
            <span>".($i+1)."</span>
          </li>\n";
    }
    $this->register_var('content', $buff . "        </ul>\n      </div>\n    </div>\n");
    $this->addBodyClass($_SESSION['topbit']);
  }

  public function randomImage($album) { // not used
    if (!$album || !$this->$album) {
      $album = $this->albumlist;
      shuffle($album);
      $album = $album[0];
    }
    $images = $this->$album->images;
    shuffle($images);
    $thumbpath = thumbpath($album, $images[0]);
    return array("$album/{$images[0]}", $thumb, $this->$album->captions[$images[0]], $album, $this->$album->title, $images[0]);
  }

  public function titleSort($a, $b) {
    return $this->$a->title > $this->$b->title ? 1 : -1;
  }

  public function gen() {
    if (!$this->album) return;
    if ($this->currentpage && $this->currentpage !== $_SESSION['oldcurrentpage']) {
      $_SESSION['oldcurrentpage'] = $this->currentpage;
      $this->addHeadCSS("#bar div:hover ol {\n  top:-".(($this->currentpage - 1) * 1.3)."em;\n}");
    }
    if (filemtime('settings.php') > SETTINGSFILEMTIME + 1) {
      $buffer .= "ul#thumbarray li a span {\n  height:".TMAXHEIGHT."px;\n  line-height:".TMAXHEIGHT."px;\n}\n";
      $buffer .= "ul#thumbarray li {\n  width:".(TMAXWIDTH + 40)."px;\n}\n";
      $buffer .= "#view .thumbs ul li a {\n  height:".(TMAXHEIGHT + 30)."px;\n}\n";
      $css = file_get_contents('shampoo.css');
      file_put_contents('shampoo.css' , substr($css, 0, strpos($css, '/* GENERATED STUFF */', filesize('shampoo.css') - 300)) . "/* GENERATED STUFF */\n" . $buffer);
      $settings = file_get_contents('settings.php');
      $settings = preg_replace("/define\('SETTINGSFILEMTIME', \d+/", "define('SETTINGSFILEMTIME', " . time(), $settings);
      file_put_contents('settings.php', $settings);
    }
  }

  public function addHeadCSS($css) {
    $curcss = $this->cache['headCSS'];
    if ($curcss) {
      $curcss = str_replace('<style type=\"text/css\">', '', $curcss);
      $curcss = str_replace('</style>', '', $curcss);
      $curcss = "\n  " . trim($curcss);
    }
    $css = str_replace("\n", "\n    ", $css);
    $curcss = "  <style type=\"text/css\">$curcss\n    $css\n  </style>\n";
    $this->register_var('headCSS', $curcss);
  }

  public function addBodyClass($class) {
    if (!$class) return;
    $cur = rtrim($this->cache['bodyclass'], '"');
    if (!$cur) $cur = ' class="' . $class . '"';
    else $cur .= " $class\"";
    $this->register_var('bodyclass', $cur, true);
  }

  public function makeHeadLink($rel, $href, $title) {
    if ($this->cache['links']) $s = '  ';
    $this->register_var('links', "$s<link rel=\"$rel\" href=\"$href\" title=\"$title\">\n");
  }
  
  public function get_password($album) {
    $pwfile = (array)@file('.pw');
    if (!$album) {
      $bits = (array)parseLine($pwfile[0]);
      return $bits[0] && $bits[1] && $bits[0] === '!!adminpw' ? $bits[1] : '';
    }
    else {
      foreach ($pwfile as $line) {
        $bits = parseLine($line);
        if ($bits[0] === $album) return $bits[1];
      }
    }
    return false;
  }

  public function register_var($name, $value, $overwrite = false) {
    if ($overwrite) $this->cache[$name] = $value;
    else $this->cache[$name] .= $value;
  }

  public function output($template, $direct = false) {
    if (!$direct) $template = file_get_contents($template) or exit("<br><b>Script stopped</b>: Check that the template file exists ($template) and that it is in the right directory (the same one as " . basename(__FILE__) . ')');
    $r = preg_match_all('/{(\w+)}/', $template, $matches);
    foreach ($matches[1] as $match) {
      $template = str_replace("{{$match}}", rtrim($this->cache[$match]), $template);
    }
    $template = preg_replace("/\n\s*\n/m", "\n", $template);
    echo utf8_encode($template);
  }
}

/* FUNCTIONS */

  function makePrettyTitle($str) {
    $str[0] = mb_strtoupper($str[0]);
    return $str;
  }

  function ampquot($str) {
    return str_replace(array('&', '"'), array('&amp;', '&quot;'), $str);
  }

  function dateSort($a, $b) {
    return filemtime($a) < filemtime($b) ? 1 : -1;
  }

  function truncate($string) {
    if (strlen($string) > 70) return array_shift(explode("\n", wordwrap($string, 70))) . '...';
    return $string;
  }

  function parseLine($line) {
    $line = rtrim(utf8_decode(ltrim($line, "\xef\xbb\xbf"))); // BOM must be removed, otherwise first line gets skipped
    if (empty($line) || strpos($line, "//") === 0) return false;
    return array_map('trim', explode("|", $line));
  }

  function realUrl($url) {
    return substr_replace($url, '.', strrpos($url, '_'), 1);
  }

  function fakeUrl($url) {
    return substr_replace($url, '_', strrpos($url, '.'), 1);
  }

  function strip_ext($str) {
    return substr($str, 0, strrpos($str, '.'));
  }

  function isValidImage($image) {
    $ext = strrchr($image, '.');
    if ($ext === '.txt') return false;
    if (($ext === '.flv' || $ext === '.mp4') && function_exists('isValidVideo')) return isValidVideo($image);
    $a = @getimagesize($image);
    if (($a[2] > 0 && $a[2] < 4) || strpos($image, 'bmp') > 0) return true;
  }

  function createThumb($album, $img, $temp = false) {
    $ext = strrchr($img, '.');
    if (!is_dir('thumbs')) mkdir('thumbs');
    $thumbext = $ext !== '.gif' && $ext !== '.png' ? '.jpg' : $ext;
    $dir = $temp ? 'thumbs/temp/' : "thumbs/$album/";
    $imagepath = $temp ? $dir . $img : "$album/$img";
    $thumbpath = $temp ? $dir . strip_ext($img) . '_thumb' . $thumbext : $dir . strip_ext($img) . $thumbext;
    if (file_exists($thumbpath)) {
      return $thumbpath;
    }
    $info = @getimagesize($imagepath);
    if (is_array($info) && $info[0] < TMAXWIDTH && $info[1] < TMAXHEIGHT && $info[2] < 4) {
      return $imagepath;
    }
    if (!$temp && !is_dir($dir)) {
      mkdir($dir);
    }
    if ($ext === '.mp4' || $ext === '.flv') {
      if (function_exists('makeVideoThumb')) return makeVideoThumb($album, $img);
      else return false;
    }
    if (!$temp && strpos($img, ' ') > -1) {
      $oldimg = $img;
      $img = str_replace(' ', '_', trim($img));
      $res = rename($imagepath, "$album/$img");
      if ($res !== true) $img = $oldimg;
    }
    $imgname = strip_ext($img);
    list($width, $height, $type) = $info;
    if ($width === false) return log_error("$imagepath isn't a valid image or it doesn't exist");
    if (TMAXWIDTH < $width && $width >= $height) {
      $thumbwidth = TMAXWIDTH;
      $thumbheight = ($thumbwidth / $width) * $height;
    }
    elseif (TMAXHEIGHT < $height && $height >= $width) {
      $thumbheight = TMAXHEIGHT;
      $thumbwidth = ($thumbheight /$height) * $width;
    }
    else {
      $thumbheight = $height;
      $thumbwidth = $width;
    }
    $imgbuffer = imagecreatetruecolor($thumbwidth, $thumbheight);
    switch($type) {
      case 1: $image = imagecreatefromgif($imagepath); break;
      case 2: $image = imagecreatefromjpeg($imagepath); break;
      case 3: $image = imagecreatefrompng($imagepath); break;
      case 6: $image = imagecreatefrombmp($imagepath); $ext = '.jpg'; break;
      case 15: $image = imagecreatefromwbmp($imagepath); $ext = '.jpg'; break;
      default: return log_error("Tried to create thumbnail from $imagepath: not a valid image");
    }
    if (!$image) return log_error("Image creation from $imagepath failed for an unknown reason. Probably not a valid image.");
    if ($type === 1 || $type === 3) { // preserve GIF/PNG transparency
      $transparent = imagecolortransparent($image);
      if ($transparent >= 0) {
        imagepalettecopy($image, $imgbuffer);
        imagefill($imgbuffer, 0, 0, $transparent);
        imagecolortransparent($imgbuffer, $transparent);
        imagetruecolortopalette($imgbuffer, true, 256);
      }
    }
    imagecopyresampled($imgbuffer, $image, 0, 0, 0, 0, $thumbwidth, $thumbheight, $width, $height);
    if ($type === 2) imageinterlace($imgbuffer); // jpeg only
    switch($type) {
      case 1: $success = imagegif($imgbuffer, $thumbpath); break;
      case 3: $success = imagepng($imgbuffer, $thumbpath); break;
      default: $success = imagejpeg($imgbuffer, $thumbpath, 80);
    }
    imagedestroy($imgbuffer);
    imagedestroy($image);
    return $success ? $thumbpath : false;
  }

  function getExif($image, $onlydatetime) {
    $ex = @exif_read_data($image, 0);
    if ($ex === false) return false;
    if ($ex['DateTimeOriginal']) {
      list($date, $time) = explode(' ', $ex['DateTimeOriginal']);
      $datetime = date("j M Y", strtotime(str_replace(":", "/", $date))) . ', ' . substr($time, 0, 5);
      if (!$onlydatetime) $exifdata['Taken'] = $datetime;
    }
    if ($onlydatetime) return $datetime;
    if ($ex['Width']) $exifdata['Width'] = $ex['Width'];
    if ($ex['Height']) $exifdata['Height'] = $ex['Height'];
    if ($ex['Make'] || $ex['Model']) {
      if (strpos($ex['Model'], $ex['Make']) === 0) $exifdata['Camera'] = $ex['Model'];
      else {
        if ($ex['Make']) $exifdata['Make'] = $ex['Make'];
        if ($ex['Model']) $exifdata['Model'] = $ex['Model'];
      }
    }
    if ($ex['ExposureTime']) {
      $z = explode('/', $ex['ExposureTime']);
      if ($z[0] / $z[1] < 1) $exifdata['Exposure Length'] =  sprintf('1/%d sec.', $z[1] / $z[0]);
      else $exifdata['Exposure Length'] = sprintf('%d sec.', $z[0] / $z[1]);
    }
    if ($ex['FocalLength']) {
      $z = explode('/', $ex['FocalLength']);
      $exifdata['Focal Length'] = sprintf('%.1f mm', $z[0] / $z[1]);
    }
    if ($ex['COMPUTED']['ApertureFNumber'] || $ex['ApertureFNumber'] || $ex['FNumber']) {
      if ($ex['COMPUTED']['ApertureFNumber']) $exifdata['Aperture'] = $ex['COMPUTED']['ApertureFNumber'];
      else {
        $z = $ex['FNumber'] || $ex['ApertureFNumber'];
        $z = explode('/', $z);
        $exifdata['Aperture'] = sprintf('f/%.01f', pow(2, $z[0] / $z[1] / 2));
      }
    }
    if ($ex['ISOSpeedRatings']) {
      $exifdata['ISO Speed'] = $ex['ISOSpeedRatings'];
    }
    if ($ex['Flash']) {
      switch($ex['Flash']) {
        case 0x0000:
        case 0x0010:
        case 0x0018:
        case 0x0020:
        case 0x0058: $exifdata['Flash'] = 'Not fired'; break;
        default: $exifdata['Flash'] = 'Fired';
      }
    }
    return count($exifdata) > 0 ? $exifdata : false;
  }

  function imagecreatefrombmp($filename) { // Written by D H Kold (dhkold.com)
    if (!$handle = fopen($filename, "rb")) return false; //Open file in binary mode
    //1 : Load file headers
    $file = unpack("vfile_type/Vfile_size/Vreserved/Vbitmap_offset", fread($handle, 14));
    if ($file['file_type'] != 19778) return false;

    //2 : Load BMP headers
    $bmp = unpack('Vheader_size/Vwidth/Vheight/vplanes/vbits_per_pixel'.
                  '/Vcompression/Vsize_bitmap/Vhoriz_resolution'.
                  '/Vvert_resolution/Vcolors_used/Vcolors_important', fread($handle, 40));
    $bmp['colors'] = pow(2, $bmp['bits_per_pixel']);
    if ($bmp['size_bitmap'] == 0) $bmp['size_bitmap'] = $file['file_size'] - $file['bitmap_offset'];
    $bmp['bytes_per_pixel'] = $bmp['bits_per_pixel']/8;
    $bmp['bytes_per_pixel2'] = ceil($bmp['bytes_per_pixel']);
    $bmp['decal'] = ($bmp['width']*$bmp['bytes_per_pixel'] / 4);
    $bmp['decal'] -= floor($bmp['width'] * $bmp['bytes_per_pixel'] / 4);
    $bmp['decal'] = 4 - (4 * $bmp['decal']);
    if ($bmp['decal'] == 4) $bmp['decal'] = 0;

    //3 : Load palette colours
    if ($bmp['colors'] < 16777216) {
     $palette = unpack('V' . $bmp['colors'], fread($handle, $bmp['colors']*4));
    }

    //4 : Image creation
    $img = fread($handle, $bmp['size_bitmap']);
    $vide = chr(0);

    $res = imagecreatetruecolor($bmp['width'], $bmp['height']);
    $p = 0;
    $y = $bmp['height'] - 1;
    while ($y >= 0) {
     $x = 0;
     while ($x < $bmp['width']) {
      if ($bmp['bits_per_pixel'] == 24)
         $color = unpack("V", substr($img, $p, 3) . $vide);
      elseif ($bmp['bits_per_pixel'] == 16) {
         $color = unpack("n", substr($img, $p, 2));
         $color[1] = $palette[$color[1]+1];
      }
      elseif ($bmp['bits_per_pixel'] == 8) {
         $color = unpack("n", $vide . substr($img, $p, 1));
         $color[1] = $palette[$color[1] + 1];
      }
      elseif ($bmp['bits_per_pixel'] == 4) {
         $color = unpack("n", $vide . substr($img, floor($p), 1));
         if (($p * 2) % 2 == 0) $color[1] = ($color[1] >> 4) ;
         else $color[1] = ($color[1] & 0x0F);
         $color[1] = $palette[$color[1] + 1];
      }
      elseif ($bmp['bits_per_pixel'] == 1) {
         $color = unpack("n", $vide . substr($img, floor($p), 1));
         if     (($p*8)%8 == 0) $color[1] =  $color[1]        >>7;
         elseif (($p*8)%8 == 1) $color[1] = ($color[1] & 0x40)>>6;
         elseif (($p*8)%8 == 2) $color[1] = ($color[1] & 0x20)>>5;
         elseif (($p*8)%8 == 3) $color[1] = ($color[1] & 0x10)>>4;
         elseif (($p*8)%8 == 4) $color[1] = ($color[1] & 0x8)>>3;
         elseif (($p*8)%8 == 5) $color[1] = ($color[1] & 0x4)>>2;
         elseif (($p*8)%8 == 6) $color[1] = ($color[1] & 0x2)>>1;
         elseif (($p*8)%8 == 7) $color[1] = ($color[1] & 0x1);
         $color[1] = $palette[$color[1]+1];
      }
      else return false;
      imagesetpixel($res, $x, $y, $color[1]);
      $x++;
      $p += $bmp['bytes_per_pixel'];
     }
     $y--;
     $p += $bmp['decal'];
    }
    fclose($handle);
    return $res;
  }
  
  function isValidVideo($video) {
		if(!is_readable($video)) return false;
		$res = shell_exec(FFMPEG . " -i $video 2>&1");
		preg_match('/\b(\d+x\d+)\b/', $res, $matches);
		if ($matches[1]) list($width, $height) = explode('x', $matches[1]);
		else return true;
		return array($width, $height);
	}
	
	function makeVideoThumb($album, $video) {
  	$res = shell_exec(FFMPEG . " -i $album/$video 2>&1");
		preg_match('/\b\d+x\d+\b/', $res, $matches);
		if ($matches) {
  		list($width, $height) = explode('x', $matches[0]);
		  if (TMAXWIDTH < $width && $width >= $height) {
        $thumbwidth = TMAXWIDTH;
        $thumbheight = ($thumbwidth / $width) * $height;
      }
      elseif (TMAXHEIGHT < $height && $height >= $width) {
        $thumbheight = TMAXHEIGHT;
        $thumbwidth = ($thumbheight /$height) * $width;
      }
      else {
        $thumbheight = $height;
        $thumbwidth = $width;
      }
      $thumbwidth = (int)round($thumbwidth);
      $thumbheight = (int)round($thumbheight);
      if ($thumbheight & 1) $thumbheight++; // if odd, add 1
      if ($thumbwidth & 1) $thumbwidth++;   // frame height must be even, or ffmpeg fails
      $thumb = "thumbs/$album/" . strip_ext($video) . '.jpg';
		  exec(FFMPEG . " -i $album/$video -an -vcodec png -ss 00:00:01 -r 1 -vframes 1 -s {$thumbwidth}x$thumbheight -f rawvideo -y $thumb");
		  if (!(file_exists($thumb) && filesize($thumb) > 0)) return false;
		  preg_match('/\bDuration: \d{2}:(\d{2}):(\d{2}(?:\.\d*)?)\b/', $res, $duration); // ignores hours
		  if ($duration) { // add duration to bottom corner of thumb
 	      $duration = $duration[1] . ':' . round((float)$duration[2]);
	      $vidlen = imagecreate(44, 18);
	      imagecolorallocate($vidlen, 0, 0, 0);
	      $white = imagecolorallocate($vidlen, 255, 255, 255);
	      imagettftext($vidlen, 10, 0, 3, 14, $white, THUMBFONT, $duration);
	      $imgbuffer = imagecreatefrompng($thumb);
	      imagecopy($imgbuffer, $vidlen, $thumbwidth - 46, $thumbheight - 20, 0, 0, 44, 18);
	      imageinterlace($imgbuffer);
 	      $output = imagejpeg($imgbuffer, $thumb, 80);
 	      imagedestroy($imgbuffer);
	    }
	  }
	  else { // no ffmpeg
	    $thumb = "thumbs/$album/" . strip_ext($video) . '.jpg';
  	  $imgbuffer = imagecreate(TMAXWIDTH, (int)round(TMAXHEIGHT * 0.8));
  	  imagecolorallocate($imgbuffer, 255, 255, 255);
  	  $textcolour = imagecolorallocate($imgbuffer, 100, 100, 255);
  	  if (strlen($video) > 20) $video = substr($video, 0, 8) . '...' . substr($video, -8);
  	  imagettftext($imgbuffer, 10, 0, 10, 30, $textcolour, THUMBFONT, $video);
  	  $icon = @imagecreatefrompng('img/vid.png');
  	  if ($icon) imagecopy($imgbuffer, $icon, 42, 50, 0, 0, 56, 42);
  	  $output = imagepng($imgbuffer, $thumb, 9);
      imagedestroy($imgbuffer);
  	  imagedestroy($icon);
    }
    return $output ? $thumb : false;
	}

  function log_error($error) {
    file_put_contents('errors.log', $error . "\r\n", FILE_APPEND);
    return false;
  }
?>